#include <gtk/gtk.h>
#include <webkit2/webkit2.h>

#include "glob-app-window.h"
#include "glob-store.h"

typedef struct _GlobAppWindowPrivate GlobAppWindowPrivate;

struct _GlobAppWindowPrivate
{
    GlobStore *store;
    GtkWidget *header;
    GtkWidget *webview;
    GtkWidget *treeview;
};

G_DEFINE_TYPE_WITH_PRIVATE (GlobAppWindow, glob_app_window, GTK_TYPE_APPLICATION_WINDOW)

static void
update_header (GlobStore     *store,
               gint           matches,
               GlobAppWindow *win)
{
    GlobAppWindowPrivate *priv;
    gchar *status;
    
    priv = glob_app_window_get_instance_private (win);

    status = g_strdup_printf ("Found %d matches", matches);
    gtk_header_bar_set_subtitle (GTK_HEADER_BAR (priv->header), status);

    g_free (status);
}

static void
entry_icon_press (GtkEntry             *entry,
                  GtkEntryIconPosition  icon_pos,
                  GdkEventButton       *event,
                  GlobAppWindow        *win)
{
    GlobAppWindowPrivate *priv;
    
    priv = glob_app_window_get_instance_private (win);

    gtk_list_store_clear (GTK_LIST_STORE (priv->store));
    gtk_widget_grab_focus (GTK_WIDGET (entry));
}

static void
entry_search_changed (GtkSearchEntry *entry,
                      GlobAppWindow  *win)
{
    GlobAppWindowPrivate *priv;

    priv = glob_app_window_get_instance_private (win);

    if (gtk_entry_get_text_length (GTK_ENTRY (entry)) > 0)
    {
        const gchar *pattern;

        pattern = gtk_entry_get_text (GTK_ENTRY (entry));
        glob_store_set_pattern (priv->store, pattern);
    }
}

static void
treeview_row_activated (GtkTreeView       *tree_view,
                        GtkTreePath       *path,
                        GtkTreeViewColumn *column,
                        GlobAppWindow     *win)
{
    GlobAppWindowPrivate *priv;
    GtkTreeIter iter;
    
    priv = glob_app_window_get_instance_private (win);

    if (gtk_tree_model_get_iter (GTK_TREE_MODEL (priv->store), &iter, path))
    {
        gchar     *word;
        gchar     *uri;

        gtk_tree_model_get (GTK_TREE_MODEL (priv->store), &iter, 0, &word, -1);
        uri = g_strdup_printf ("http://en.wiktionary.org/w/index.php?title=Special:Search&search=%s", word);
        webkit_web_view_load_uri (WEBKIT_WEB_VIEW (priv->webview), uri);

        g_free (word);
        g_free (uri);
    }
}

static void
glob_app_window_class_init (GlobAppWindowClass *klass)
{
    gtk_widget_class_set_template_from_resource (GTK_WIDGET_CLASS (klass),
                                                 "/uk/me/bcowan/glob/glob.ui");

    gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GlobAppWindow, header);
    gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GlobAppWindow, webview);
    gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GlobAppWindow, treeview);

    gtk_widget_class_bind_template_callback (GTK_WIDGET_CLASS (klass), entry_icon_press);
    gtk_widget_class_bind_template_callback (GTK_WIDGET_CLASS (klass), entry_search_changed);
    gtk_widget_class_bind_template_callback (GTK_WIDGET_CLASS (klass), treeview_row_activated);

    g_type_class_add_private (klass, sizeof (GlobAppWindowPrivate));
}

static void
glob_app_window_init (GlobAppWindow *win)
{
    GlobAppWindowPrivate *priv;
    
    priv = glob_app_window_get_instance_private (win);

    gtk_widget_init_template (GTK_WIDGET (win));

    priv->store = glob_store_new ();
    gtk_tree_view_set_model (GTK_TREE_VIEW (priv->treeview), GTK_TREE_MODEL (priv->store));
    g_signal_connect (priv->store, "finished",
                      G_CALLBACK (update_header), win);

}

/**
 * glob_app_window_new:
 * @app: the application object
 *
 * Gets an application window with all its widgets setup for use.
 *
 * Returns: (transfer full) an application window
 */
GtkWidget *
glob_app_window_new (GlobApplication *app)
{
    return g_object_new (GLOB_TYPE_APP_WINDOW, "application", app, NULL);
}
